﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Syncfusion.UI.Xaml.Gantt;
using Syncfusion.UI.Xaml.TreeGrid;
using Syncfusion.UI.Xaml.ScrollAxis;

// The Blank Page item template is documented at https://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409
using Key = Windows.System.VirtualKey;

namespace Peamel.JiProject.UWP
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        SfTreeGrid _ganttGrid = null;

        public MainPage()
        {
            this.InitializeComponent();
        }

        private void GanttControl_OnLoaded(object sender, RoutedEventArgs e)
        {
            this.GanttControl.ContextFlyout = new MenuFlyout();
            this.GanttControl.ContextFlyout.Opening += ContextFlyout_Opening;

            this.GanttControl.SelectionChanged += GanttControl_SelectionChanged;

            // Reflect the GanttGrid property in the GanttControl.
            PropertyInfo ganttGridPropertyInfo = this.GanttControl.GetType()
                .GetProperty("Grid", BindingFlags.Instance | BindingFlags.NonPublic);
            GanttGrid ganttGrid = ganttGridPropertyInfo.GetValue(this.GanttControl) as GanttGrid;

            _ganttGrid = ganttGrid;

            // Define the controls in this template for the custom column.
            //DataTemplate cellTemplate = this.GanttRootGrid.Resources["CellTemplate"] as DataTemplate;

            //// TreeGridTemplateColumn is used to add the custom column for the GanttGrid.
            //ganttGrid.Columns.Add(new TreeGridTemplateColumn()
            //{
            //    HeaderText = "Button",
            //    CellTemplate = cellTemplate,
            //    AllowEditing = false
            //});

        }

        private void GanttControl_SelectionChanged(object sender, Syncfusion.UI.Xaml.Gantt.SelectionChangedEventArgs e)
        {
            var k = e.Item as TaskDetail;
            int level = this.GanttControl.GetLevel(k);
        }

        Boolean isDragging = false;
        TaskDetail dropTask = null;
        TaskDetail destinationTask = null;
        int itemToMoveIndex = -1;

        private void ContextFlyout_Opening(object sender, object e)
        {
            destinationTask = this.GanttControl.SelectedTask as TaskDetail;
            var dm = this.GanttControl.ContextFlyout as MenuFlyout;
            dm.Items.Clear();

            if (isDragging == true)
            {
                //if (droppingIndex > 0)
                {
                    MenuFlyoutItem dropAboveItem = new MenuFlyoutItem();
                    dropAboveItem.Text = "Drop Above";
                    dropAboveItem.Click += DropAboveItem_Click;
                    dm.Items.Add(dropAboveItem);
                }
                //if (droppingIndex < (totalEntries-1))
                {
                    MenuFlyoutItem dropBelowItem = new MenuFlyoutItem();
                    dropBelowItem.Text = "Drop Below";
                    dropBelowItem.Click += DropBelowItem_Click;
                    dm.Items.Add(dropBelowItem);
                }

                {
                    MenuFlyoutItem cancelMoveItem = new MenuFlyoutItem();
                    cancelMoveItem.Text = "Cancel Move";
                    cancelMoveItem.Click += CancelMoveItem_Click;
                    dm.Items.Add(cancelMoveItem);
                }

                return;
            }

            // You can only move things when sorting is cleared
            if ((_ganttGrid.SortColumnDescriptions == null) || (_ganttGrid?.SortColumnDescriptions?.Count == 0))
            {
                MenuFlyoutItem menuFlyoutItem = new MenuFlyoutItem();
                menuFlyoutItem.Text = "Move";
                menuFlyoutItem.Click += MoveTask_Click;
                dm.Items.Add(menuFlyoutItem);
            }
            else
            {
                MenuFlyoutItem clearSorttItem = new MenuFlyoutItem();
                clearSorttItem.Text = "Clear Sort";
                clearSorttItem.Click += ClearSorttItem_Click;
                dm.Items.Add(clearSorttItem);
            }

            MenuFlyoutItem refreshGridtItem = new MenuFlyoutItem();
            refreshGridtItem.Text = "Refresh";
            refreshGridtItem.Click += RefreshGridtItem_Click; ;
            dm.Items.Add(refreshGridtItem);

        }

        private void RefreshGridtItem_Click(object sender, RoutedEventArgs e)
        {
            GanttControl.Refresh();
        }

        private void ClearSorttItem_Click(object sender, RoutedEventArgs e)
        {
            _ganttGrid.SortColumnDescriptions.Clear();
            GanttControl.Refresh();
        }

        private void CancelMoveItem_Click(object sender, RoutedEventArgs e)
        {
            isDragging = false;
            itemToMoveIndex = -1;
        }        

        private void DropBelowItem_Click(object sender, RoutedEventArgs e)
        {
            TreeNode selectedNode = _ganttGrid.View.CurrentItem as TreeNode;
            TaskDetail selectedTaskDetail = this.GanttControl.SelectedTask as TaskDetail;
            int droppingIndex = -1;

            // ----- PERFORM THE INSERT ------

            // Check if the select item has is a parent
            if (selectedNode.HasChildNodes == true)
            {
                // It is a parent, so we add the moving item as one of its children
                // Since this is drop below, we add it to the beginning of the list
                selectedTaskDetail.Children.Insert(0, dropTask);
                droppingIndex = 0;
            }
            else
            {
                // The current selection is not a parent, but it may have a parent
                if (selectedNode.ParentNode == null)
                {
                    // No parent, so just find the index in the main tree
                    int index = ViewModel.TaskCollection.IndexOf(selectedTaskDetail);
                    droppingIndex = index + 1;
                    ViewModel.TaskCollection.Insert(droppingIndex, dropTask);
                }
                else
                {
                    // The current selection is a child
                    TaskDetail parentTaskDetail = selectedNode?.ParentNode?.Item as TaskDetail;

                    int index = parentTaskDetail.Children.IndexOf(selectedTaskDetail);
                    droppingIndex = index + 1;
                    parentTaskDetail.Children.Insert(droppingIndex, dropTask);
                }
            }

            // ----- PERFORM THE DELETE ------

            // If the have the same parent, then we need to validate
            // if the insertion would have changed the index 
            if (parentOfMovingTask == selectedNode?.ParentNode?.Item)
            {
                if (itemToMoveIndex > droppingIndex)
                {
                    itemToMoveIndex++;
                }
            }

            if (parentOfMovingTask == null)
            {
                // There is no parent of the moving task, so just delete it
                // from the root collection
                ViewModel.TaskCollection.RemoveAt(itemToMoveIndex);
            }
            else
            {
                // It has a parent, so delete it as a child of the parent
                //parentOfMovingTask.Children.RemoveAt(itemToMoveIndex);
                parentOfMovingTask.Children.Remove(dropTask);
            }

            // Moving complete, clean everytying up
            GanttControl.Refresh();
            isDragging = false;
            parentOfMovingTask = null;
            itemToMoveIndex = -1;
        }

        private void DropAboveItem_Click(object sender, RoutedEventArgs e)
        {
       
            //droppingIndex = this.GanttControl.SelectedIndex;

            //// Check if we are below where we are deleting.
            //// Because we first remove the item, if we drop above the item
            //// we are removing, we need to decrement the dropping index
            //if (droppingIndex > itemToMoveIndex)
            //{
            //    droppingIndex = droppingIndex - 1;
            //}

            //ViewModel.TaskCollection.RemoveAt(itemToMoveIndex);
            //ViewModel.TaskCollection.Insert(droppingIndex, dropTask);

            //isDragging = false;
            //itemToMoveIndex = -1;
            //droppingIndex = -1;
        }

        TaskDetail parentOfMovingTask = null;

        private void MoveTask_Click(object sender, RoutedEventArgs e)
        {
            isDragging = true;
            dropTask = this.GanttControl.SelectedTask as TaskDetail;
            TreeNode node = _ganttGrid.View.CurrentItem as TreeNode;
            parentOfMovingTask = node.ParentNode?.Item as TaskDetail;
            if (parentOfMovingTask == null)
            {
                // the index is from the root node
                itemToMoveIndex = ViewModel.TaskCollection.IndexOf(dropTask);
            }
            else
            {
                // It's a child of a parent
                // We find the index within the children
                itemToMoveIndex = node.ParentNode.ChildNodes.IndexOf(node);
            }
        }

        private void GanttGrid_DragStarting(UIElement sender, DragStartingEventArgs args)
        {
            throw new NotImplementedException();
        }

        private void GanttGrid_DragEnter(object sender, DragEventArgs e)
        {
            int i = 0;
        }
    }

    public class TaskDetailExtended : TaskDetail, IGanttTask
    {
        public TaskDetailExtended()
        {
            key = "TBD";
        }

        public String key { get; set; }
    }

    public class ViewModel
    {
        static public ObservableCollection<TaskDetail> TaskCollection
        {
            get;
            set;
        }

        public ViewModel()
        {
            TaskCollection = new ObservableCollection<TaskDetail>();

            var task200 = new TaskDetail
            {
                StartDate = new DateTime(2010, 6, 9),
                FinishDate = new DateTime(2010, 6, 9),
                Name = "Task 2",
                ID = "2"
            };

            var task210 = new TaskDetail
            {
                StartDate = new DateTime(2010, 6, 9),
                FinishDate = new DateTime(2010, 6, 9),
                Name = "Task 2.1",
                ID = "2.1"
            };

            var task220 = new TaskDetail
            {
                StartDate = new DateTime(2010, 6, 9),
                FinishDate = new DateTime(2010, 6, 9),
                Name = "Task 2.2",
                ID = "2.2"
            };

            TaskCollection.Add(new TaskDetail { StartDate = new DateTime(2010, 6, 2), FinishDate = new DateTime(2010, 6, 5), Duration = 0, Name = "Task 1", ID = "1" });
            task200.Children.Add(task210);
            task200.Children.Add(task220);
            TaskCollection.Add(task200);

            var task30 = new TaskDetail { StartDate = new DateTime(2010, 6, 10), FinishDate = new DateTime(2010, 6, 12), Name = "Task 3.0", ID = "3.0" };
            var task31 = new TaskDetail { StartDate = new DateTime(2010, 6, 10), FinishDate = new DateTime(2010, 6, 12), Name = "Task 3.1", ID = "4.0"};
            
            TaskRelationship tr = new TaskRelationship();
            tr.ID = "2";
            tr.Relationship = Relationship.FinishToStart;
            task31.Predecessors.Add(tr);

            task30.Children.Add(task31);
            TaskCollection.Add(task30);
            TaskCollection.Add(new TaskDetail { StartDate = new DateTime(2010, 6, 9), FinishDate = new DateTime(2010, 6, 9), Name = "Task 4", ID = "4" });
        }
    }


}
